@@ -31,6 +31,13 @@ class Operation(Enum):
     VISIT = 4
     CHANGE = 5
     DEL = 6
+    WALK = 7
+
+class Committed(Enum):
+    CREATED = 0
+    UPDATED = 1
+    DELETED = 2
+    SAVED = 3
 
 class MetaProps(Enum):
     @property
